# -*- encoding: utf-8 -*-

# Author: M. Sakano (Wise Babel Ltd)

require 'db_suit_rails/sql_skelton/col_index'

$stdout.sync=true
$stderr.sync=true

#################################################
# Unit Test
#################################################

#if $0 == __FILE__
  gem "minitest"
  # require 'minitest/unit'
  require 'minitest/autorun'
  # MiniTest::Unit.autorun

  class TestUnitSqlSkeltonColIndex < MiniTest::Test
    T = true
    F = false
    SCFNAME = File.basename(__FILE__)
    CI = SqlSkelton::ColIndex

    def setup
      # @ib = 1
    end
    # teardown is not often used.
    def teardown
      # @foo = nil
    end


    def test_initialize01
      c1 = CI.new('tblold')
      assert_equal ['tblold'],  c1.colmaps.keys
      c2 = CI.new(['tblold'])
      assert_equal ['tblold'],  c2.colmaps.keys
      c3 = CI.new()
      c3.push('tblold')
      assert_equal ['tblold'],  c3.colmaps.keys
      c3.push('tblold2')
      assert_equal %w(tblold tblold2),  c3.colmaps.keys
    end

    def test_push01
      oldtbl = 'tblold1'
      oldcol = 'tblcol1'
      ci = CI.new(oldtbl)
      assert_raises(DbSuitRailsError){ ci.push(oldtbl) }
      ci.push(oldtbl, oldcol, 'atai')
      assert_raises(DbSuitRailsError){ ci.push(oldtbl, oldcol) }

      assert  ci.table_exist?(oldtbl)
      assert  ci.column_registered?(oldtbl, oldcol)
      assert !ci.table_exist?('naiyo')
      assert !ci.column_registered?('naiyo', oldcol)
      assert !ci.column_registered?(oldtbl, 'naiyo')

      assert_equal 'atai', ci.newcolval(oldtbl, oldcol)
      ci.set_newcolval_with(oldtbl, oldcol, nil)
      assert_nil           ci.newcolval(oldtbl, oldcol)
    end

    def test_fkey01
      oldtbl = 'tblold1'
      oldcol = 'tblcol1'
      ci = CI.new(oldtbl)
      assert  !ci.fkey?(oldtbl, oldcol)

      ci.push(oldtbl, oldcol, fkey: true)
      assert   ci.fkey?(oldtbl, oldcol)
      assert_equal true, ci.fkeyval(oldtbl, oldcol)

      ci.set_fkeyval_with(oldtbl, oldcol, nil)
      assert  !ci.fkey?(oldtbl, oldcol)
    end

    def test_update01
      oldtbls = %w(tblold1 tblold2 tblold3)
      oldcols = %w(colold1 colold2 colold21)
      ci = CI.new(oldtbls[0])

      ci.push(oldtbls[0], oldcols[0])
      ci.push(oldtbls[0], oldcols[1])

      assert_raises(DbSuitRailsError){ ci.paircolnames('naiyo') }
      assert_equal oldcols[0..1], ci.oldcolnames(oldtbls[0])

      assert_raises(DbSuitRailsError){ ci.update(oldtbls[1], oldcols[0]) }
      assert_raises(DbSuitRailsError){ ci.update(oldtbls[0], oldcols[2]) }

      assert_nil               ci.newcolval(oldtbls[0], oldcols[0])
      ci.update(oldtbls[0], oldcols[0])
      assert_equal oldcols[0], ci.newcolval(oldtbls[0], oldcols[0])

      # Test update!() - wrapper of push()
                                       ci.update!(oldtbls[0], oldcols[2])
      assert_equal oldcols,    ci.oldcolnames(oldtbls[0])
      assert_equal oldcols[2], ci.newcolval(oldtbls[0], oldcols[2])
      ci.update!(oldtbls[2], oldcols[0])  # Push for new table.

      # Test update!() - wrapper of update()
      ci.update!(oldtbls[0], oldcols[1])
      assert_equal oldcols[1], ci.newcolval(oldtbls[0], oldcols[1])
      # Test newcolnames()
      assert_equal oldcols,    ci.newcolnames(oldtbls[0])

      # Test unique_colname
      assert_equal 'n1_'+oldcols[1], ci.instance_eval{ unique_colname(oldtbls[0], oldcols[0], oldcols[1], useprefix: true) }
      assert_equal oldcols[1]+'2',   ci.instance_eval{ unique_colname(oldtbls[0], oldcols[0], oldcols[1], useprefix: false) }
      assert_equal oldcols[1],       ci.instance_eval{ unique_colname(oldtbls[0], oldcols[1], useprefix: false) }
      # Reserved words: 'type' => 'type_cli',  'MyTable_count' => 'MyTable_count_cli'
      ['type', oldtbls[0]+'count'].each do |i|
        ci.push(oldtbls[0], i)
        assert_equal i+'_cli',   ci.instance_eval{ unique_colname(oldtbls[0], i, useprefix: false) }
      end
    end


    def test_newcolnames01
      oldtbls = %w(tblold1 tblprt2 tblprt3)
      oldcols = %w(id non_id non_id_cli job_id normal)
      ci = CI.new(oldtbls)
      oldtbl  = oldtbls[0]
      oldcols.each do |ec|
        ci.push(oldtbl, ec, ec) 
      end 
      assert_equal oldcols,              ci.newcolnames(oldtbl)
      assert_equal [nil]+oldcols[1..-1], ci.newcolnames(oldtbl, exclude: 'id', compact: false)
      assert_equal oldcols[1..-1],       ci.newcolnames(oldtbl, exclude: 'id', compact: true)
    end

    def test_mk_newcolname01
      oldtbls = %w(tblold1 tblprt2 tblprt3)
      oldcols = %w(id non_id non_id_cli job_id normal)
      ci = CI.new(oldtbls)

      oldtbl  = oldtbls[0]
      oldcols.each do |ec|
        ci.push(oldtbl, ec, ec) 
      end 
      assert_equal oldcols,       ci.newcolnames(oldtbl)
      assert_equal 'non_id_cli1', ci.instance_eval{ mk_newcolname(oldtbl, oldcols[1]) }

      ci.update(oldtbl, oldcols[1])
      ## update does nothing, because everything is already set and fkey is nil.
      assert_equal oldcols,       ci.newcolnames(oldtbl)
      assert(    'non_id_cli1' != ci.newcolval(oldtbl, oldcols[1]))

      ## force option
      ci.update(oldtbl, oldcols[1], force: true)
      assert_equal 'non_id_cli1', ci.newcolval(oldtbl, oldcols[1])

      ## update!
      ci.update( oldtbl, oldcols[3])
      assert(    'job_id_cli' != ci.newcolval(oldtbl, oldcols[3]))
      ci.update!(oldtbl, oldcols[3])
      assert_equal 'job_id_cli', ci.newcolval(oldtbl, oldcols[3])

      ## update! with :all
      assert(      'id_cli'   != ci.newcolval(oldtbl, oldcols[0]))
      ci.update!(oldtbl, :all)
      assert(      'id_cli'   == ci.newcolval(oldtbl, oldcols[0]))
      assert_equal 'id_cli',     ci.newcolval(oldtbl, oldcols[0])
      assert_equal 'job_id_cli', ci.newcolval(oldtbl, oldcols[3]) # unchanged
      # p ci.newcolnames(oldtbl)
      #  => ["id_cli", "non_id_cli2", "non_id_cli", "job_id_cli1", "normal"]

      ## update! with fkey is set
      fk_job = SqlSkelton::Fkey.new('tblold1', oldcols[3], 'tblprt2', 'eid')
      ci.set_fkeyval_with(oldtbl, oldcols[3], fk_job)
      assert_equal 'job_id',     ci.instance_eval{ mk_newcolname(oldtbl, oldcols[3]) }

      assert_equal 'job_id_cli', ci.newcolval(oldtbl, oldcols[3])
      ci.update(oldtbl, oldcols[3], fkey: fk_job)
      assert_equal 'job_id',     ci.newcolval(oldtbl, oldcols[3])

      ## Is non_id_cli1 reversed if fkey is set?
      fk_non = SqlSkelton::Fkey.new('tblold1', oldcols[1], 'tblprt2', 'eid')
      assert_equal 'non_id_cli1', ci.newcolval(oldtbl, oldcols[1])
      ci.update!(oldtbl, oldcols[1], fkey: fk_non)
      assert_equal 'non_id',      ci.newcolval(oldtbl, oldcols[1])

      ## Is id properly set if fkey is set?
      fk_id  = SqlSkelton::Fkey.new('tblold1', oldcols[0], 'tblprt2', 'eid')
      assert_equal 'id_cli',    ci.newcolval(oldtbl, oldcols[0])
      ci.update!(oldtbl, oldcols[0], fkey: fk_id)
      assert_equal 'id_cli_id', ci.newcolval(oldtbl, oldcols[0])
    end

    def test_updated_col01
      oldtbls = %w(tblold1 tblprt2 tblprt3)
      oldcols = %w(id non_id non_id_cli job_id normal)
      ci = CI.new()

      assert_equal 'id_cli', ci.updated_col!(oldtbls[0], oldcols[0])

      ## updated_col! not updates if the record exists.
      ci.push(oldtbls[0], oldcols[3])
      assert(    'job_id_cli' != ci.updated_col!(oldtbls[0], oldcols[3]))

      ## update! does update.
      ci.update!(oldtbls[0], oldcols[3])
      assert_equal 'job_id_cli', ci.updated_col!(oldtbls[0], oldcols[3])
    end 

    def test_update02
      ci = CI.new()
      hs = {"reha_b01"=>{:order=>["id", "tax_id"], "id"=>{:name=>"id_cli", :fkey=>nil}, "tax_id"=>{:name=>"tax_id_cli", :fkey=>nil}}}
      ci.instance_eval{ @colmaps = hs }
      fk_now = SqlSkelton::Fkey.new("reha_b01", 'tax_id', "tax_table", "vat_id")

      ci.update!("reha_b01", nil, fkey: nil)  # fkey: fk_now  would issues a WARNING as expected.
      assert_equal hs, ci.colmaps

      ci.update!("reha_b01", 'id_cli', fkey: fk_now)
    end
  end	# class TestUnitSqlSkeltonColIndex < MiniTest::Test

#end	# if $0 == __FILE__


