# -*- encoding: utf-8 -*-

# Author: M. Sakano (Wise Babel Ltd)

require 'db_suit_rails/sql_skelton/tbl_index'

$stdout.sync=true
$stderr.sync=true
# print '$LOAD_PATH=';p $LOAD_PATH

#################################################
# Unit Test
#################################################

#if $0 == __FILE__
  gem "minitest"
  # require 'minitest/unit'
  require 'minitest/autorun'
  # MiniTest::Unit.autorun

  # For test use.
  NewTmpClass = true

  class TestUnitSqlSkeltonTblIndex < MiniTest::Test
    T = true
    F = false
    SCFNAME = File.basename(__FILE__)
    TI = SqlSkelton::TblIndex

    def setup
      # @ib = 1
    end
    # teardown is not often used.
    def teardown
      # @foo = nil
    end


    def test_initialize01
      c1 = TI.new('tblold')
      assert_equal ['tblold'],  c1.tblmaps.keys
      c2 = TI.new(['tblold'])
      assert_equal ['tblold'],  c2.tblmaps.keys
      c3 = TI.new()
      c3.push('tblold')
      assert_equal ['tblold'],  c3.tblmaps.keys
      c3.push('tblold2')
      assert_equal %w(tblold tblold2),  c3.tblmaps.keys
    end

    def test_push01
      oldtbl = 'tblold1'
      ci = TI.new(oldtbl)
      assert_raises(DbSuitRailsError){ ci.push(oldtbl) }

      assert  ci.table_exist?(oldtbl)
      assert !ci.table_exist?('naiyo')

      ci.set_newtblval_with(oldtbl, nil)
      assert_nil           ci.newtblval(oldtbl)
    end

    def test_update01
      oldtbls = %w(tblold1  tblold2  tblold3)
      newtbls = %w(tblold1s tblold2s tblold3s)
      ci = TI.new(oldtbls[0])

      ci.push(oldtbls[1])

      assert_equal oldtbls[0..1], ci.oldtblnames()
      assert_equal oldtbls[0..0]+[nil], ci.oldtblnames(exclude: oldtbls[1], compact: false)
      assert_equal oldtbls[0..0],       ci.oldtblnames(exclude: oldtbls[1], compact: true)

      assert_raises(DbSuitRailsError){ ci.update('naiyo') }

      assert_nil                   ci.newtblval(oldtbls[0])
      ci.update(oldtbls[0])
      assert_equal oldtbls[0]+'s', ci.newtblval(oldtbls[0])  # Pluralized

      # Test update()
      ci.set_newtblval_with(oldtbls[1], 'tekitoh')
      assert_equal 'tekitoh',      ci.newtblval(oldtbls[1])
                                   ci.update(   oldtbls[1])
      assert_equal 'tekitoh',      ci.newtblval(oldtbls[1])
                                   ci.update(   oldtbls[1], force: true)
      assert_equal oldtbls[1]+'s', ci.newtblval(oldtbls[1])  # Pluralized

      # Test update!() - wrapper of push()
                                   ci.update!(  oldtbls[2])
      assert_equal oldtbls[2]+'s', ci.newtblval(oldtbls[2])  # Pluralized

      # Test update!() - wrapper of update()
      ci.set_newtblval_with(oldtbls[1], 'tekitoh')
      assert_equal 'tekitoh',      ci.newtblval(oldtbls[1])
                                   ci.update!(  oldtbls[1])
      assert_equal oldtbls[1]+'s', ci.newtblval(oldtbls[1])  # Pluralized

      # Test newtblnames()
      assert_equal newtbls,        ci.newtblnames()

      # Test update!(:all) - wrapper of update()
      ci.set_newtblval_with(oldtbls[1], 'tekitoh')
      assert_equal 'tekitoh',      ci.newtblval(oldtbls[1])
      assert      (newtbls !=      ci.newtblnames())
                                   ci.update!(:all)
      assert_equal oldtbls[1]+'s', ci.newtblval(oldtbls[1])  # Pluralized
      assert_equal newtbls,        ci.newtblnames()
      assert_equal newtbls[0..1],  ci.newtblnames(exclude: oldtbls[2], compact: true)
    end


    def test_mk_newtblname01
      oldtbls = %w(study study_id_seq study_pkey studyunique string objects new_tmp_classes)
      newtbls = %w(studies studies_id_seq studies_pkey studiesunique string_clis object_clis new_tmp_class_clis)
      ci = TI.new(oldtbls)

      assert_equal [nil]*oldtbls.size, ci.newtblnames()
      oldtbls.each_index do |i| 
        assert_equal newtbls[i], ci.instance_eval{ mk_newtblname(oldtbls[i]) }
      end

      ci.update!()
      assert_equal newtbls,      ci.newtblnames()
    end

    def test_updated_tbl01
      oldtbls = %w(study study_id_seq study_pkey studyunique)
      newtbls = %w(studies studies_id_seq studies_pkey studiesunique)
      ci = TI.new()

      assert_equal newtbls[0], ci.updated_tbl!(oldtbls[0])

      ## updated_tbl! not updates if the record exists.
      ci.push(oldtbls[1])
      assert(    newtbls[1] != ci.updated_tbl!(oldtbls[1]))

      ## update! does update.
      ci.update!(oldtbls[1])
      assert_equal newtbls[1], ci.updated_tbl!(oldtbls[1])
    end
  end	# class TestUnitSqlSkeltonTblIndex < MiniTest::Test

#end	# if $0 == __FILE__


