# -*- encoding: utf-8 -*-

# Author: M. Sakano (Wise Babel Ltd)

require 'tempfile'
require 'db_suit_rails/sql_skelton'

$stdout.sync=true
$stderr.sync=true
# print '$LOAD_PATH=';p $LOAD_PATH

#################################################
# Unit Test
#################################################

#if $0 == __FILE__
  gem "minitest"
  # require 'minitest/unit'
  require 'minitest/autorun'
  # MiniTest::Unit.autorun

  class TestUnitSqlSkelton < MiniTest::Test
    T = true
    F = false
    SCFNAME = File.basename(__FILE__)
    SS = SqlSkelton

    def setup
      @tmpioin = Tempfile.new
      @tmpioin.sync=true 

      @tmpioin.puts <<EOF

--
-- Name: shop_b01; Type: TABLE; Schema: public; Owner: seller
--

CREATE TABLE public.shop_b01 (
    id integer NOT NULL,
    tax_id bigint varying(50) NOT NULL REFERENCES tax_table(vat_id),
    office_id character varying(50) NOT NULL,
    file_path character varying(300) NOT NULL
);

ALTER TABLE shop_b01 OWNER TO seller;

--
-- Name: shop_b01_id_seq; Type: SEQUENCE; Schema: public; Owner: seller
--

CREATE SEQUENCE shop_b01_id_seq
    START WITH 1
    CACHE 1;

ALTER TABLE shop_b01_id_seq OWNER TO seller;

--
-- Name: shop_b01 id; Type: DEFAULT; Schema: public; Owner: seller
--

ALTER TABLE ONLY shop_b01 ALTER COLUMN id SET DEFAULT nextval('shop_b01_id_seq'::regclass);

--
-- Data for Name: shop_b01; Type: TABLE DATA; Schema: public; Owner: seller
--

COPY shop_b01 (id, office_id, file_path) FROM stdin;
1	5	012ABCD	/ab1/20181111010101/JAPAN	SHOPSKU01/X0.d/01
2	6	012XYZA	/ab2/20181111010101/NIHON	SHOPSKU01/X9.d/02
\\.

--
-- Name: shop_b01_id_seq; Type: SEQUENCE SET; Schema: public; Owner: seller
--

SELECT pg_catalog.setval('shop_b01_id_seq', 24, true);

--
-- Name: shop_b01 shop_b01_pkey; Type: CONSTRAINT; Schema: public; Owner: seller
--

ALTER TABLE ONLY shop_b01
    ADD CONSTRAINT shop_b01_pkey PRIMARY KEY (id);
ALTER TABLE ONLY shop_b01
    ADD CONSTRAINT shop_b01_qkey PRIMARY KEY (id, tax_id, office_id);

--
-- Name: shop_b01 shop_b01unique; Type: CONSTRAINT; Schema: public; Owner: seller
--

ALTER TABLE ONLY shop_b01
    ADD CONSTRAINT shop_b01unique UNIQUE (office_id, file_path);

--
-- Name: shop_b01 tg01; Type: TRIGGER; Schema: public; Owner: seller
--

CREATE TRIGGER tg01 BEFORE INSERT OR UPDATE ON public.shop_b01 FOR EACH ROW EXECUTE PROCEDURE tg_ins_upd_trriger();

EOF
    end

    def teardown
      @tmpioin.close
    end


    def test_initialize01
      s1 = SS.new(@tmpioin.path)

      assert ( /_rails_db\.sql$/ =~ s1.outfile )
#print "DEBUG:"; p s1.instance_eval{ @strall }
      assert ( /^ALTER/ =~ s1.instance_eval{ @strall } )
      assert_equal 'mapping.csv',     s1.mappingcsv
    end

    def test_get_foreign_keys01
      s1 = SS.new(@tmpioin.path)
      strin = 'FOREIGN KEY (b, c) REFERENCES other_table (c1, c2)'
      (fkeys, strout) = s1.get_foreign_keys(strin, 'this_t')
      assert_equal 'this_t',     fkeys[0].child[:tbl]
      assert_equal 'b',          fkeys[0].child[:col]
      assert_equal 'c',          fkeys[1].child[:col]
      assert_equal 'other_table', fkeys[0].parent[:tbl]
      assert_equal 'c1',          fkeys[0].parent[:col]
      assert_equal 'other_table', fkeys[1].parent[:tbl]
      assert_equal 'c2',          fkeys[1].parent[:col]
      assert_equal strin.sub(/_table/, '_tables'), strout

      (fkeys, strout) = s1.get_foreign_keys('naiyo', 'this_t')
      assert_nil            fkeys
      assert_equal 'naiyo', strout
    end

    def test_get_foreign_keys02
      s1 = SS.new(@tmpioin.path)
      strin = ' REFERENCES products (product_no),'
      (fkeys, strout) = s1.get_foreign_keys(strin, 'this_t', 'col_old3')
      assert_equal 1, fkeys.size
      assert_equal 'this_t',     fkeys[0].child[:tbl]
      assert_equal 'col_old3',   fkeys[0].child[:col]
      assert_equal 'products',   fkeys[0].parent[:tbl]
      assert_equal 'product_no', fkeys[0].parent[:col]
      assert_equal strin, strout
    end

    def test_get_hsflag_in_read01
      s1 = SS.new(@tmpioin.path)
      hs = {
        :in_create => nil,
        :in_comment => false,
        :in_sentence => false,
        :from_stdin => false,
        :tbl_cur => nil,
      }
      assert_equal hs, s1.get_hsflag_in_read()
      s='tbl8'
      hs2 = hs.merge({:in_create => s})
      assert_equal s, hs2[:in_create]
      assert_equal hs.merge({:in_create => s}), s1.get_hsflag_in_read(in_create: s)

      assert_equal true,  s1.get_hsflag_in_read("abc",     in_sentence: true)[:in_sentence]
      assert_equal false, s1.get_hsflag_in_read("abc; \n", in_sentence: true)[:in_sentence]
      assert_equal true,  s1.get_hsflag_in_read("a\\; \n", in_sentence: true)[:in_sentence]
      assert_equal false, s1.get_hsflag_in_read("\\\\; ",  in_sentence: true)[:in_sentence]
    end

    def test_read01
      s1 = SS.new(@tmpioin.path)
      assert_match(/^\s*ADD CONSTRAINT/ , s1.instance_eval{ @strall })
      assert(      /office_id_cli/     !~ s1.instance_eval{ @strall })
      assert  s1.col_index.empty?
# open('/tmp/a.sql', 'w'){|iow| iow.print File.read(@tmpioin.path)}  ## DEBUG

      str_refac = s1.read(stage: :refactoring, setstr: false)
      assert(      /^\s*ADD CONSTRAINT/ !~ str_refac)
      assert(      /office_id_cli/      !~ str_refac)
      assert  s1.col_index.empty?, "debug: #{s1.col_index.inspect}"
# open('/tmp/b.sql', 'w'){|iow| iow.print str_refac}  ## DEBUG

      assert_raises(DbSuitRailsError){ s1.read(stage: :indexing) } # b/c refactoring is not made.
      str_indexed = s1.read(stage: :indexing, instr: str_refac)
      assert_equal "id_cli",        s1.col_index.newcolval("shop_b01", "id")
      assert_equal "office_id_cli", s1.col_index.newcolval("shop_b01", "office_id")
      assert(      /^\s*ADD CONSTRAINT/ !~ str_indexed)
      assert(      /id_cli/             !~ str_indexed)
      assert(      s1.instance_eval{@strall} != str_indexed)
# open('/tmp/c.sql', 'w'){|iow| iow.print str_indexed}  ## DEBUG

      ## Now, redoing (with setstr option is true implicitly) so the instance variable @strall is set.
      s1.read(stage: :refactoring)
      s1.read(stage: :indexing)
      assert_equal s1.instance_eval{@strall},   str_indexed

      str_final = s1.read(stage: :final, setstr: false)
      assert_match(/id_cli/, str_final)
# open('/tmp/d.sql', 'w'){|iow| iow.print str_final}  ## DEBUG

      assert s1.tbl_index.tblmaps.key?('shop_b01')
      assert_equal s1.tbl_index.tblmaps['shop_b01'], 'shop_b01s'
      assert_equal 3, s1.tbl_index.tblmaps.keys.size, sprintf("Table keys = %s", s1.tbl_index.tblmaps.keys.inspect)

      str_tmp = str_final.split("\n").grep(/ALTER TABLE/).join("\n")
      assert_match(/^-- ALTER TABLE .+ PRIMARY KEY \(id_cli\)/, str_tmp)
      assert_match(   /^ALTER TABLE .+ UNIQUE \(office_id_cli, file_path\)/, str_tmp)

      # Tests the "*_id" column name acceptable as it is.
      assert_match(   /^ALTER TABLE .+ UNIQUE \(id_cli, tax_id, office_id_cli\)/, str_tmp)

      hst = { 'shop_b01' => 'shop_b01s',
              "tax_table"=>"tax_tables",
              'shop_b01_id_seq' => 'shop_b01s_id_seq', }
            # 'shop_b01_pkey'   => 'shop_b01s_pkey',
            # 'shop_b01unique'  => 'shop_b01sunique', }
      assert_equal hst, s1.tbl_index.tblmaps

      fk_tax = SqlSkelton::Fkey.new("shop_b01", 'tax_id', "tax_table", "vat_id")
      hsc = { 'shop_b01' =>
              { :order=>%w(id tax_id office_id file_path),
                "id"       =>{:name=>"id_cli",        :fkey=>nil},
                "tax_id"   =>{:name=>"tax_id",        :fkey=>fk_tax},  # due to Foreign key, _id is preserved.
                "office_id"=>{:name=>"office_id_cli", :fkey=>nil},
                "file_path"=>{:name=>"file_path",     :fkey=>nil},
              },
              "tax_table" =>
              { :order=>%w(vat_id),
                "vat_id"   =>{:name=>"vat_id_cli",    :fkey=>nil},
              } }
      assert_equal hsc.keys.sort, s1.col_index.colmaps.keys.sort
      assert_equal hsc, s1.col_index.colmaps
    #      id integer NOT NULL,
    #office_id character varying(50) NOT NULL,
    #file_path character varying(300) NOT NULL,

    end

    def test_run01
      s1 = SS.new(@tmpioin.path, '/tmp/out.sql', mappingcsv: '/tmp/mapping.csv')
      s1.run()
    end
  end	# class TestUnitSqlSkelton < MiniTest::Test

#end	# if $0 == __FILE__


